/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * LXQt - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2012 Razor team
 * Authors:
 *   Luís Pereira <luis.artur.pereira@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#ifndef LXQTPANELLIMITS_H
#define LXQTPANELLIMITS_H

#define PANEL_DEFAULT_SIZE 32
#define PANEL_MINIMUM_SIZE 16
#define PANEL_MAXIMUM_SIZE 200
#define PANEL_HIDE_SIZE 4

#define PANEL_DEFAULT_ICON_SIZE 22
#define PANEL_DEFAULT_LINE_COUNT 1

#define PANEL_DEFAULT_BACKGROUND_COLOR "#CCCCCC"

#define PANEL_HIDE_DELAY 500
#define PANEL_HIDE_FIRST_TIME (5000 - PANEL_HIDE_DELAY)

#define PANEL_SHOW_DELAY 0

#define SETTINGS_SAVE_DELAY 3000
#endif // LXQTPANELLIMITS_H
